<?php
// $Id: term_relations.admin.inc,v 1.1 2010/11/23 13:52:51 chaps2 Exp $

/**
 * @file
 * Administration code including private implementations of hooks.
 */

/**
 * Administration settings for term relations.
 */
function term_relations_admin_settings() {
  // this setting should be elsewhere
  $form['term_relations_directional'] = array(
    '#type' => 'checkbox',
    '#title' => t('Treat term relationships as directed'),
    '#default_value' => variable_get('term_relations_directional', FALSE),
    '#description' => t('If you have a need for directed term relationships, enable this option. When disabled, the direction of term relationships may not be preserved.'),
  );

  return system_settings_form($form);
}

/**
 * Private implementation of hook_form_FORM_ID_alter().
 * Populates related terms form element with relationships that originate from
 * this term.
 */
function _term_relations_form_taxonomy_form_term_alter(&$form, &$form_state) {
  $edit = $form['#term'];
  $vocabulary = $form['#vocabulary'];

  if (!variable_get('taxonomy_override_selector', FALSE)) {
    if (term_relations_is_directed()) {
      // replace selected relations with left to right relationships only
      $l2r_related_terms = term_relations_get_related($edit['tid'], 'tid', 'l2r', $vocabulary['vid']);
      $form['advanced']['relations'] = _taxonomy_term_select(t('Related terms'), 'relations', array_keys($l2r_related_terms), $vocabulary['vid'], NULL, 1, '<'. t('none') .'>', array($edit['tid']));
      // core taxonomy_save_term() will delete right to left relations.
      // save right to left relations in form to restore after form submission.
      // don't specify vid as form will have deleted all r2l relationships from all vocabularies
      $form['advanced']['#r2l_relations'] = term_relations_get_related($edit['tid'], 'tid', 'r2l');

      $form['#submit'][] = 'term_relations_form_taxonomy_form_term_submit';
    }
  }
}

/**
 * Extra submit handler for the standard term edit form - taxonomy_form_term.
 */
function term_relations_form_taxonomy_form_term_submit($form, &$form_state) {
  if (term_relations_is_directed()) {
    if (!empty($form['advanced']['#r2l_relations'])) {
      // now replace the r2l relationships that have been deleted by taxonomy_term_save().
      foreach ($form['advanced']['#r2l_relations'] as $related_tid => $related_term) {
        if ($related_tid != 0) {
          db_query('INSERT INTO {term_relation} (tid1, tid2) VALUES (%d, %d)', $related_tid, $form_state['tid']);
        }
      }
    }
  }
}
